<?php
// $Author: noah $
// $Revision: 0.0 $
// $Date: 2004/10/24 22:12:53 $
//
// This defines common file management primitive functions 
//     browse_directory
//     list_files 
//     list_dirs 
//     filepath2url 
//     get_filename_extension 
//     mkdir_p
//     copyr
//     rmdirr
//     mode_num
//     check_uploaded_file_for_errors 
//     valid_filename 
//     handle_upload 

function browse_directory($cwd, $show_subdirs=0)
// This returns HTML that can be used to display a directory listing.
{
	global $CONFIG;
	// Current Working Directory must be a subdirectory of upload_home
	// and it must be a real directory.
	if ( ! is_dir($cwd) || $cwd != strstr($cwd, $CONFIG["upload_home"]))
	{
		echo "<h1>Invalid folder</h1>\n";
		echo "<blockquote>$cwd</blockquote>\n";
		return;
	}

	if ($show_subdirs)
	{	
		echo "<h1>Current Folder</h1>\n";
		echo "<blockquote><b>" . basename ($cwd) . "</b> (<i>$cwd</i>)</blockquote>\n";
		
		echo "<h1>Folders under Current Folder</h1>\n";
		echo "<blockquote>\n";
		$dirs = list_dirs($cwd);
		$parent_dir = dirname ($cwd);
		if ($cwd != $CONFIG["upload_home"])
			echo "<a href=\"fileman.php?command=cd&dir=$parent_dir\">.. UP</a><br>\n";
		foreach ($dirs as $dirname)
		{
			echo "<a href=\"fileman.php?command=cd&dir=$cwd/$dirname\">$dirname</a>";
			echo "<br>\n";
		}
		echo "</blockquote>\n";
		echo "<h1>Files in Current Folder</h1>\n";
	}
	$files = list_files ($cwd);
	echo "<blockquote>\n";
	if (count($files)>0)
	{
		echo '<table cellSpacing="0" cellPadding="3" border="0">';
		echo '<tr>';
		echo '<td class="report">Command</td>';
		echo '<td class="report">File Name</td>';
		echo '<td class="report">Size</td>';
		echo '<td class="report">Date</td>';
		echo '</tr>';
		foreach ($files as $filename)
		{
			$url = filepath2url($cwd . "/" . $filename);
			echo "<tr>";
			echo "<td>";
			echo "<a href=\"fileman.php?command=del&filename=$cwd/$filename\">DELETE</a>";
			echo "&nbsp;-&nbsp;";
			echo "<a href=\"download.php?filename=$cwd/$filename\">DOWNLOAD</a>";
			echo "</td>";
			echo "<td>";
			echo "<a href=\"$url\">$filename</a>";
			echo "</td>";
			echo "<td align=\"right\">";
			echo filesize ($cwd . "/" . $filename);
			echo "</td>";
			echo "<td>";
			echo date ("Y-m-d H:i:s", filemtime($cwd . "/" . $filename));
			echo "</td>";
			echo "</tr>\n";
		}
		echo '</table>';
	}
	else
	{
		echo "<i>NO FILES</i>\n";
	}
	echo "</blockquote>\n";
}

function list_files ($repository_path)
// This returns a list of files that are in the given path
{
    $d = dir($repository_path);
    $file_list = array();
    while (false !== ($entry = $d->read()))
    {
        if ( ! is_file($repository_path . "/" . $entry) )
        {
            continue;
        }
        $file_list [] = $entry;
    }
    $d->close();
    return $file_list;
}

function list_dirs ($repository_path)
// This returns a list of directories that are in the given path
{
    $d = dir($repository_path);
    $dir_list = array();
    while (false !== ($entry = $d->read()))
    {
        if ($entry == '.' || $entry == '..')
        {
            continue;
        }
        if ( ! is_dir($repository_path . "/" . $entry))
        {
            continue;
        }
        $dir_list [] = $entry;
    }
    $d->close();
    return $dir_list;
}

function filepath2url ($filepathname)
// This tries to convert a filesystem path to an URL path.
// This assumes that the file path is accessable under the root URL.
// If the file is NOT accessable then you will NOT get an error.
// You will get an invalid URL if the file is not accessable.
{
	global $CONFIG;
	$root_url = "http://".$_SERVER['HTTP_HOST'];
	$filepathname_parts = explode ("/", $filepathname);
	$filename = basename ($filepathname);
	$upload_home_parts = explode ("/", $_SERVER['DOCUMENT_ROOT']);
	$path_diff = array_values (array_diff ($filepathname_parts, $upload_home_parts));
	$file_url = implode("/", $path_diff);
	return ($root_url . "/" . $file_url);
}

function get_filename_extension ($filename)
// This returns the extension (suffix) of a filename.
// That is everything after the last "." dot.
{
   $pos = strrpos($filename, '.');
   if (false !== $pos && strlen($filename) > $pos + 1)
   {
       $ext = substr($filename, $pos + 1);
   }
   else
   {
       $ext = "";
   }
   return $ext;
}

function mkdir_p($target)
// This makes a directory and all intermediate directories.
// This is like "mkdir -p".
{
   if (file_exists($target) && !is_dir($target))
       return 0;                          // failure
   if (substr($target, 0, 1) != '/')
       $target = "/".$target;             // prepend a slash
   if (ereg('\.\.', $target) || ereg("[^a-zA-Z0-9\._\-\/]", $target))
       return 0;                          // if illegal dirname
   if (mkdir($target))                    // if new dir created,
       return 1;                          // we are successful

   if (mkdir_p(substr($target, 0, (strrpos($target, '/')))) == 1)
       return (mkdir_p($target));
   else
       return 0;
}

function copyr($source, $dest)
// This is a recursive copy. This copies directories and files.
{
    // Simple copy for a file
    if (is_file($source))
    {
        return copy($source, $dest);
    }

    // Make destination directory
    if (!is_dir($dest))
    {
        mkdir($dest);
    }

    // Loop through the folder
    $dir = dir($source);
    while (false !== $entry = $dir->read())
    {
        // Skip pointers
        if ($entry == '.' || $entry == '..')
        {
            continue;
        }

        // Deep copy directories
        if (is_dir("$source/$entry") && ($dest !== "$source/$entry"))
        {
            copyr("$source/$entry", "$dest/$entry");
        }
        else
        {
            copy("$source/$entry", "$dest/$entry");
        }
    }

    // Clean up
    $dir->close();
    return true;
}

function rmdirr($dirname)
// This recursivly deletes directories and files. This is like "rm -rf *"
{
    // Sanity check
    if (!file_exists($dirname))
    {
        return false;
    }

    // Simple delete for a file
    if (is_file($dirname))
    {
        return unlink($dirname);
    }

    // loop through this folder
    $dir = dir($dirname);
    while (false !== $entry = $dir->read())
    {
        // skip parent and self links.
        if ($entry == '.' || $entry == '..')
        {
            continue;
        }

        if (is_dir("$dirname/$entry"))
        {
            // recurse directories      
            rmdirr("$dirname/$entry");
        }
        else
        {
            // delete files
            unlink("$dirname/$entry");
        }
    }

    $dir->close();
    return rmdir($dirname);
}


function mode_num($mode) 
// This converts chmod style strings to a number representation.
// For example: passing $mode as 'rwxr-xr--' returns 754.
// This doesn't do sticky bits.
{
    $mode = str_pad($mode,9,'-');
    $trans = array('r'=>4,'w'=>2,'x'=>1,'-'=>0);
    $newmode = $trans[$mode[0]]+$trans[$mode[1]]+$trans[$mode[2]];
    $newmode = $newmode << 3;
    $newmode += $trans[$mode[3]]+$trans[$mode[4]]+$trans[$mode[5]];
    $newmode = $newmode << 3;
    $newmode += $trans[$mode[6]]+$trans[$mode[7]]+$trans[$mode[8]];
    return $newmode;
}

function check_uploaded_file_for_errors ($file)
{
    if ( ! is_uploaded_file($file))
    {
        if ($file['error'] == 1)
        {
             echo "File error:<blockquote>\n";
             echo "The file you are trying to upload is too big (limited by upload_max_filesize in php.ini).";
             echo "</blockquote>\n";
             return True;
        }
        if ($file['error'] == 2)
        {
             echo "File error:<blockquote>\n";
             echo "The file you are trying to upload is too big (limited by MAX_FILE_SIZE).";
             echo "</blockquote>\n";
             return True;
        }
        if ($file['error'] == 3)
        {
             echo "File error:<blockquote>\n";
             echo "The file you are trying upload was only partially uploaded.";
             echo "</blockquote>\n";
             return True;
        }
        if ($file['error'] == 4)
        {
             echo "File error:<blockquote>\n";
             echo "You did not select a file for upload.";
             echo "</blockquote>\n";
             return True;
        }
    }
    return False;
}

function valid_filename ($filename)
// This returns True if the filename is valid.
{
	$good_chars = "[- _A-Za-z0-9+,./:~%?=#&@)(]";
	$bad_chars = ereg_replace($good_chars, "", $filename);
	if (strlen($bad_chars) > 0)
	{
		// echo "Filename is invalid.\n";
		return False; 
	}
	return True;
}

function handle_upload ($homedir)
{
    $file = array_shift($_FILES);
    if (check_uploaded_file_for_errors($file) == True)
    {
            return Null;
    }
    else
    {
		$filename = $homedir . "/" . $file['name'];
		if ( ! valid_filename ($filename))
		{
			return Null;
		}
        if (is_file ($filename)) // delete old file if it exists
        {
            unlink ($filename);
        }
        if (move_uploaded_file ($file['tmp_name'], $filename) )
        {
            return $filename;
        }
    }
    return Null;
}

// This takes care of PHP's magic quotes madness.
set_magic_quotes_runtime(0);
if (get_magic_quotes_gpc())
{
   function magic_quotes_die_die_die ($element)
   {
       if (is_array($element))
           return array_map("magic_quotes_die_die_die", $element);
       else
           return stripslashes($element);
   }

   $_GET     = array_map("magic_quotes_die_die_die", $_GET);
   $_POST    = array_map("magic_quotes_die_die_die", $_POST);
   $_COOKIE  = array_map("magic_quotes_die_die_die", $_COOKIE);
   $_REQUEST = array_map("magic_quotes_die_die_die", $_REQUEST);
}
?>
